#!/bin/sh
#
# Prepare archive with test fixtures for tree-sitter unit-tests
#
# The upstream CI is using `master` to fetch the grammars; see `script/fetch-fixtures`.
# Replace with the exact commits or tags to make reproducible.
# Make sure to check the licenses for all the projects we download here.
set -e

DESTDIR="$(dirname "$0" | xargs readlink -f)"
VERSION=$(rpmspec --query --qf '%{version}' --srpm "$DESTDIR/rust-tree-sitter-cli.spec")
MODE=fetch

while [ "$#" -gt 0 ]; do
    case $1 in
        [0-9]*)
            VERSION="$1";;
        --update)
            MODE=update;;
        *)
            break;;
    esac
    shift
done

if [ -z "$VERSION" ]; then
    echo "Cannot determine the tree-sitter-cli version"
    exit 1
fi

OUTFILE="tree-sitter-fixtures-$VERSION.tar.xz"
# WORKDIR="$(mktemp -d tree-sitter-XXXX)"
WORKDIR="${PWD}/tree-sitter-temp-$VERSION"

extract() {
    test -d "$2" || mkdir -p "$2";
    tar xaf "$1" -C "$2" --strip-components=1
}

get () { # args: github_repo, commit, destination
    FILENAME="$(basename "$1")-$2.tar.gz"
    OUT_DIR=${3:-$(basename "$1")-$2}
    echo "Fetching $1@$2 to $OUT_DIR"
    if [ ! -f "$WORKDIR/$FILENAME" ]; then
        curl -LSs  -o "$WORKDIR/$FILENAME" "https://github.com/$1/archive/$2/$FILENAME"
    fi
    rm -rf "$OUT_DIR"
    extract "$WORKDIR/$FILENAME" "$OUT_DIR"
}

# Get date of the commit (tag, sha or any other ref)
get_commit_date () { # args: github_repo, ref
    curl -LSs -H "Accept: application/vnd.github.v3+json" \
            "https://api.github.com/repos/$1/commits/$2" |
        jq -r '.commit.committer.date'
}

# Get last commit in the branch made before the specified timestamp
get_commit_by_date () { # args: github_repo, timestamp, ref (optional)
    curl -LSs -H "Accept: application/vnd.github.v3+json" \
            "https://api.github.com/repos/$1/commits?per_page=1&until=$2${3:+&sha=$3}" |
        jq -r '.[0].sha'
}

# Get test repository commits at the timestamp of the tree-sitter release tag
# (i.e. the exact test data the release tag was tested on)
#
# GitHub API is rate limted, so don't abuse this mode
if [ "$MODE" = "update" ]; then
    TS_TAG_DATE=$(get_commit_date tree-sitter/tree-sitter "v$VERSION")
    echo "# tree-sitter tag date: $TS_TAG_DATE"
    echo "#             LANGUAGE          BRANCH                      CACHED REVISION"

    fetch_grammar () { # args: language, branch, cached revision
        COMMIT=$(get_commit_by_date "tree-sitter/tree-sitter-$1" "$TS_TAG_DATE" "$2")
        printf "fetch_grammar %-17s %-27s %s\n" "$1" "$2" "$COMMIT"
    }
else
    mkdir -p "$WORKDIR"
    cd "$WORKDIR"

    # License: MIT
    get tree-sitter/tree-sitter "v$VERSION" "tree-sitter-$VERSION"
    cd "tree-sitter-$VERSION"

    fetch_grammar () { # args: language, branch, cached revision
        get "tree-sitter/tree-sitter-$1" "${3:-${2:-master}}" "test/fixtures/grammars/$1"
    }
fi

# Verified that all the repositories referenced below are licensed under MIT
# and provide LICENSE files - 2022-04-01
# tree-sitter tag date: 2022-03-03T04:50:52Z
#             LANGUAGE          BRANCH                      CACHED REVISION
fetch_grammar bash              master                      275effdfc0edce774acf7d481f9ea195c6c403cd
fetch_grammar c                 master                      e348e8ec5efd3aac020020e4af53d2ff18f393a9
fetch_grammar cpp               master                      a403969cbb82e53a703ca4fe4a3586ad39817348
fetch_grammar embedded-template master                      d21df11b0ecc6fd211dbe11278e92ef67bd17e97
fetch_grammar go                master                      0fa917a7022d1cd2e9b779a6a8fc5dc7fad69c75
fetch_grammar html              master                      161a92474a7bb2e9e830e48e76426f38299d99d1
fetch_grammar java              master                      8a6ab8e8adb119bc35b133040077b60f6cf539ea
fetch_grammar javascript        partial-order-precedences   4c2f62903210181ededf461aaa82ca1f804e13e4
fetch_grammar jsdoc             master                      189a6a4829beb9cdbe837260653b4a3dfb0cc3db
fetch_grammar json              master                      203e239408d642be83edde8988d6e7b20a19f0e8
fetch_grammar php               master                      0ce134234214427b6aeb2735e93a307881c6cd6f
fetch_grammar python            master                      78c4e9b6b2f08e1be23b541ffced47b15e2972ad
fetch_grammar ruby              master                      daa17aeb69995788432a11bcc52cd78a0f768ca5
fetch_grammar rust              master                      8e7bae52db18f589d5fd80e91f54ccd2f0b5e7c4
fetch_grammar typescript        master                      e8e8e8dc2745840b036421b4e43286750443cb13

if [ "$MODE" != "update" ]; then
    echo "Writing source archive $DESTDIR/$OUTFILE"
    # Drop the examples from the grammar repos. Don't want to check the licenses for unused files
    tar caf "$DESTDIR/$OUTFILE" --exclude=examples -v LICENSE test/fixtures
fi
